@@ -119,9 +119,12 @@ class WidgetBase(Callback.Callbacks):
         # this is for compatibility with Qt widgets
         pass
 
+    def set_border_width(self, pix):
+        self.widget.set_border_width(pix)
 
 # BASIC WIDGETS
 
+
 class TextEntry(WidgetBase):
     def __init__(self, text='', editable=True):
         super(TextEntry, self).__init__()
@@ -1018,13 +1021,26 @@ class TreeView(WidgetBase):
         treeselection = self.tv.get_selection()
         treeselection.unselect_all()
 
-    def select_path(self, path, state=True):
+    def get_selected_paths(self):
         treeselection = self.tv.get_selection()
-        item = self._path_to_item(path)
-        if state:
-            treeselection.select_iter(item)
-        else:
-            treeselection.unselect_iter(item)
+        model, pathlist = treeselection.get_selected_rows()
+        paths = []
+        for path in pathlist:
+            item = model.get_iter(path)
+            paths.append(self._get_path(item))
+        return paths
+
+    def select_paths(self, paths, state=True):
+        treeselection = self.tv.get_selection()
+        for path in paths:
+            item = self._path_to_item(path)
+            if state:
+                treeselection.select_iter(item)
+            else:
+                treeselection.unselect_iter(item)
+
+    def select_path(self, path, state=True):
+        self.select_paths([path], state=state)
 
     def highlight_path(self, path, onoff, font_color='green'):
         item = self._path_to_item(path)  # noqa
@@ -1109,7 +1125,8 @@ class TreeView(WidgetBase):
             column, cell, model, iter = args[:4]
             bnch = model.get_value(iter, 0)
             if isinstance(bnch, str):
-                cell.set_property('text', bnch)
+                if not isinstance(cell, Gtk.CellRendererPixbuf):
+                    cell.set_property('text', bnch)
             elif isinstance(bnch, GdkPixbuf.Pixbuf):
                 cell.set_property('pixbuf', bnch)
             elif isinstance(bnch[idx], GdkPixbuf.Pixbuf):
@@ -1123,7 +1140,8 @@ class TreeView(WidgetBase):
             column, cell, model, iter = args[:4]
             bnch = model.get_value(iter, 0)
             if isinstance(bnch, str):
-                cell.set_property('text', '')
+                if not isinstance(cell, Gtk.CellRendererPixbuf):
+                    cell.set_property('text', '')
             elif isinstance(bnch, GdkPixbuf.Pixbuf):
                 cell.set_property('text', '')
             elif isinstance(bnch[idx], GdkPixbuf.Pixbuf):
@@ -1192,9 +1210,6 @@ class ContainerBase(WidgetBase):
         # TODO: can this be made more accurate?
         self.widget.set_border_width(left)
 
-    def set_border_width(self, pix):
-        self.widget.set_border_width(pix)
-
 
 class Box(ContainerBase):
     def __init__(self, orientation='horizontal'):
@@ -1781,8 +1796,7 @@ class GridBox(ContainerBase):
     def __init__(self, rows=1, columns=1):
         super(GridBox, self).__init__()
 
-        # TODO: GtkTable has been deprecated--migrate to GtkGrid?
-        w = Gtk.Table(rows=rows, columns=columns)
+        w = Gtk.Grid()
         self.widget = w
         self.tbl = {}
         self.num_rows = rows
@@ -1791,13 +1805,14 @@ class GridBox(ContainerBase):
     def resize_grid(self, rows, columns):
         self.num_rows = rows
         self.num_cols = columns
-        self.widget.resize(rows, columns)
+        # No resize() for Gtk Grid widget
+        #self.widget.resize(rows, columns)
 
     def set_row_spacing(self, val):
-        self.widget.set_row_spacings(val)
+        self.widget.set_row_spacing(val)
 
     def set_column_spacing(self, val):
-        self.widget.set_col_spacings(val)
+        self.widget.set_column_spacing(val)
 
     def set_spacing(self, val):
         self.set_row_spacing(val)
@@ -1826,18 +1841,7 @@ class GridBox(ContainerBase):
         self.tbl[key] = child
         self.add_ref(child)
         w = child.get_widget()
-        if stretch > 0:
-            xoptions = (Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.SHRINK |
-                        Gtk.AttachOptions.FILL)
-            yoptions = (Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.SHRINK |
-                        Gtk.AttachOptions.FILL)
-        else:
-            xoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-            yoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-        self.widget.attach(w, col, col + 1, row, row + 1,
-                           xoptions=xoptions, yoptions=yoptions,
-                           xpadding=0, ypadding=0)
-        self.widget.show_all()
+        self.widget.attach(w, row, col, 1, 1)
         self.make_callback('widget-added', child)
 
     def remove(self, child, delete=False):
@@ -1860,42 +1864,11 @@ class GridBox(ContainerBase):
 
         self.resize_grid(self.num_rows + 1, self.num_cols)
 
-        xoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-        yoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-
-        # handle case where user inserts row before the end of the gridbox
-        if index < self.num_rows:
-            # shift key/value pairs down to make the row empty at index
-            for i in range(self.num_rows - 1, index - 1, -1):
-                for j in range(self.num_cols):
-                    key = (i, j)
-                    if key in self.tbl:
-                        child = self.tbl.pop(key)
-                        self.tbl[(i + 1, j)] = child
-                        # move actual widget down in GtkTable
-                        w = child.get_widget()
-                        self._remove(w)
-                        row, col = i + 1, j
-                        self.widget.attach(w, col, col + 1, row, row + 1,
-                                           xoptions=xoptions, yoptions=yoptions,
-                                           xpadding=0, ypadding=0)
-
-        for j in range(self.num_cols):
-            child = widgets[j]
-            # populate inserted row with widgets
-            self.tbl[(index, j)] = child
-            self.add_ref(child)
-            row, col = index, j
-            w = child.get_widget()
-            self.widget.attach(w, col, col + 1, row, row + 1,
-                               xoptions=xoptions, yoptions=yoptions,
-                               xpadding=0, ypadding=0)
-
-        self.num_rows += 1
-        self.widget.show_all()
-
-        for child in widgets:
-            self.make_callback('widget-added', child)
+        if index > self.num_rows:
+            index = self.num_rows
+        self.widget.insert_row(index)
+        for col, child in enumerate(widgets):
+            self.add_widget(child, index, col)
 
     def append_row(self, widgets):
         num_rows, num_cols = self.get_row_column_count()
@@ -1905,9 +1878,6 @@ class GridBox(ContainerBase):
         if index < 0 or index >= self.num_rows:
             raise ValueError("Index ({}) out of bounds ({})".format(index, self.num_rows))
 
-        xoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-        yoptions = (Gtk.AttachOptions.FILL | Gtk.AttachOptions.SHRINK)
-
         # remove widgets in row from table
         for j in range(self.num_cols):
             key = (index, j)
@@ -1915,25 +1885,8 @@ class GridBox(ContainerBase):
                 child = self.tbl.pop(key)
                 self.remove(child)
 
-        if index < self.num_rows - 1:
-            # if not removing very last row,
-            # shift dict key, value pairs up
-            for i in range(index + 1, self.num_rows):
-                for j in range(self.num_cols):
-                    key = (i, j)
-                    if key in self.tbl:
-                        child = self.tbl.pop(key)
-                        self.tbl[(i - 1, j)] = child
-                        # move actual widget up in GtkTable
-                        w = child.get_widget()
-                        self._remove(w)
-                        row, col = i - 1, j
-                        self.widget.attach(w, col, col + 1, row, row + 1,
-                                           xoptions=xoptions, yoptions=yoptions,
-                                           xpadding=0, ypadding=0)
-
+        self.widget.remove_row(index)
         self.num_rows -= 1
-        self.resize_grid(self.num_rows, self.num_cols)
 
 
 class Toolbar(ContainerBase):
@@ -2558,6 +2511,8 @@ def make_widget(title, wtype):
         w = TextEntrySet()
     elif wtype == 'combobox':
         w = ComboBox()
+    elif wtype == 'comboboxedit':
+        w = ComboBox(editable=True)
     elif wtype in ('spinbox', 'spinbutton'):
         w = SpinBox(dtype=int)
     elif wtype == 'spinfloat':
@@ -2613,17 +2568,18 @@ def hadjust(w, orientation):
 
 
 def build_info(captions, orientation='vertical'):
-    vbox = Gtk.VBox(spacing=2)
+    box = Box(orientation=orientation)
+    box.set_spacing(2)
 
     numrows = len(captions)
     numcols = reduce(lambda acc, tup: max(acc, len(tup)), captions, 0)
     if (numcols % 2) != 0:
         raise ValueError("Column spec is not an even number")
     numcols = int(numcols // 2)
-    table = Gtk.Table(rows=numrows, columns=numcols)
-    table.set_row_spacings(2)
-    table.set_col_spacings(4)
-    vbox.pack_start(table, False, False, 0)
+    table = GridBox(rows=numrows, columns=numcols)
+    table.set_row_spacing(2)
+    table.set_column_spacing(4)
+    box.add_widget(table, stretch=0)
 
     wb = Bunch.Bunch()
     row = 0
@@ -2638,18 +2594,12 @@ def build_info(captions, orientation='vertical'):
                 else:
                     name = name_mangle('lbl_' + title[:-1])
                 w = make_widget(title, wtype)
-                table.attach(w.get_widget(), col, col + 1, row, row + 1,
-                             xoptions=Gtk.AttachOptions.FILL,
-                             yoptions=Gtk.AttachOptions.FILL,
-                             xpadding=1, ypadding=1)
+                table.add_widget(w, row, col)
                 wb[name] = w
             col += 1
         row += 1
 
-    vbox.show_all()
-
-    w = wrap(vbox)
-    w = hadjust(w, orientation=orientation)
+    w = hadjust(box, orientation=orientation)
 
     return w, wb
 
