@@ -80,15 +80,33 @@ plugins = [
     Bunch(module='Operations', workspace='operations', start=True,
           hidden=True, category='System', menu="Operations [G]",
           ptype='global', enabled=True),
-    Bunch(module='Toolbar', workspace='toolbar', start=True,
-          hidden=True, category='System', menu="Toolbar [G]", ptype='global',
+    Bunch(module='Toolbar', klass='Toolbar', workspace='toolbar',
+          start=True, hidden=True, category='System', ptype='global',
           enabled=True),
+    Bunch(module='Toolbar', klass='Toolbar_Ginga_Image',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
+    Bunch(module='Toolbar', klass='Toolbar_Ginga_Plot',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
+    Bunch(module='Toolbar', klass='Toolbar_Ginga_Table',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
     Bunch(module='Pan', workspace='uleft', start=True,
           hidden=True, category='System', menu="Pan [G]", ptype='global',
           enabled=True),
     Bunch(module='Info', tab='Synopsis', workspace='lleft', start=True,
           hidden=True, category='System', menu="Info [G]", ptype='global',
           enabled=True),
+    Bunch(module='Info', klass='Info_Ginga_Image',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
+    Bunch(module='Info', klass='Info_Ginga_Plot',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
+    Bunch(module='Info', klass='Info_Ginga_Table',
+          hidden=True, category='System', ptype='local',
+          enabled=True, exclusive=False),
     Bunch(module='Thumbs', tab='Thumbs', workspace='right', start=True,
           hidden=True, category='System', menu="Thumbs [G]", ptype='global',
           enabled=True),
@@ -363,10 +381,13 @@ class ReferenceViewer(object):
         logger = log.get_logger(name=logname, options=options)
 
         if options.basedir is not None:
+            # command line option overrules
             self.basedir = os.path.expanduser(options.basedir)
         if self.basedir is not None:
-            paths.ginga_home = self.basedir
+            # custom basedir
+            paths.set_home(self.basedir)
         else:
+            # stock ginga basedir
             self.basedir = paths.ginga_home
 
         # Get settings (preferences)
