@@ -13,12 +13,12 @@ import binascii
 from collections import namedtuple
 from io import BytesIO
 
+from PIL import Image
 import tornado.web
 import tornado.websocket
 import tornado.template
 
 from ginga.misc import Bunch, Callback
-from ginga.util.io import io_rgb
 from ginga.util.evloop import get_ioloop
 from ginga.fonts import font_asst
 
@@ -261,7 +261,7 @@ def get_image_src_from_buffer(img_buf, imgtype='png'):
 
 
 def get_icon(iconpath, size=None, format='png'):
-    image = io_rgb.PILimage.open(iconpath)
+    image = Image.open(iconpath)
     if size is not None:
         wd, ht = size
     else:
@@ -276,7 +276,7 @@ def get_icon(iconpath, size=None, format='png'):
 
 
 def get_native_image(imgpath, format='png'):
-    image = io_rgb.PILimage.open(imgpath)
+    image = Image.open(imgpath)
 
     img_buf = BytesIO()
     image.save(img_buf, format=format)
